use crate::db::schema::DollProfile;
use sqlx::types::chrono;

use super::schema::{CreateDollProfile, DollTagsDb};

pub async fn get(mut db: DollTagsDb, ident: i64) -> sqlx::Result<Option<DollProfile>> {
	sqlx::query_as!(
		DollProfile,
		r#"
			select
			id, name, pronouns, handler_name, handler_url, kind, breed, behaviour, description, chassis_type, chassis_id, chassis_color,
			microchipped as "microchipped!: bool",
			created_at as "created_at!: chrono::NaiveDateTime", updated_at as "updated_at!: Option<chrono::NaiveDateTime>"
			from doll_profiles where id = ?
		"#,
		ident
	)
	.fetch_optional(&mut **db).await
}

pub async fn create(mut db: DollTagsDb, doll: CreateDollProfile<'_>) -> sqlx::Result<i64> {
	sqlx::query!(
		r#"
			insert into doll_profiles
			(id, name, pronouns, handler_name, handler_url, kind, breed, behaviour, description, chassis_type, chassis_id, chassis_color, microchipped)
			values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
		"#,
		doll.id,
		doll.name,
		doll.pronouns,
		doll.handler_name,
		doll.handler_url,
		doll.kind,
		doll.breed,
		doll.behaviour,
		doll.description,
		doll.chassis_type,
		doll.chassis_id,
		doll.chassis_color,
		doll.microchipped,
	).execute(&mut **db).await?;

	Ok(doll.id)
}
