#[macro_use]
extern crate rocket;
use db::doll;
use db::schema::{CreateDollProfile, DollTags, DollTagsDb};
use regex::Regex;
use rocket::form::Form;
use rocket::fs::{relative, FileServer};
use rocket::response::Redirect;
use rocket_db_pools::Database;
use rocket_dyn_templates::{context, Template};

pub mod db;
pub mod rng;

pub fn id_public_to_db(id: &str) -> Option<i64> {
	let id_re = Regex::new(r"^\d{6}$").unwrap();

	if id_re.is_match(id) {
		id.parse::<i64>().ok()
	} else {
		None
	}
}
pub fn id_db_to_public(id: i64) -> String {
	format!("{:0>6}", id)
}

#[get("/")]
fn index() -> Template {
	Template::render("index", context! {})
}

#[get("/register")]
fn show_register() -> Template {
	Template::render("register", context! {})
}

// TODO: Validation
#[derive(Debug, FromForm)]
struct TagForm<'a> {
	pub ident: &'a str,

	pub name: &'a str,
	pub pronoun_subject: &'a str,
	pub pronoun_object: &'a str,
	pub pronoun_possessive: &'a str,

	pub handler_name: &'a str,
	pub handler_link: &'a str,

	pub kind: &'a str,
	pub breed: &'a str,
	pub behaviour: &'a str,
}

#[post("/register", data = "<tag>")]
async fn handle_register(db: DollTagsDb, tag: Form<TagForm<'_>>) -> Template {
	println!("register: {:?}", tag);
	let id = id_public_to_db(&tag.ident).expect("wrong fmt");
	let pronouns = format!(
		"{}/{}/{}",
		tag.pronoun_subject, tag.pronoun_object, tag.pronoun_possessive
	);

	doll::create(
		db,
		CreateDollProfile {
			id,
			name: tag.name,
			pronouns: pronouns.as_str(),
			handler_name: tag.handler_name,
			handler_url: if tag.handler_link.len() != 0 {
				Some(tag.handler_link)
			} else {
				None
			},
			kind: if tag.kind.len() != 0 {
				Some(tag.kind)
			} else {
				None
			},
			breed: if tag.breed.len() != 0 {
				Some(tag.breed)
			} else {
				None
			},
			behaviour: if tag.behaviour.len() != 0 {
				Some(tag.behaviour)
			} else {
				None
			},
			description: None,
			chassis_type: None,
			chassis_id: None,
			chassis_color: None,
			microchipped: false,
		},
	)
	.await
	.expect("fuck");

	Template::render("register", context! {})
}

#[get("/profile?<ident>")]
async fn show_profile(db: DollTagsDb, ident: &str) -> Result<Template, Redirect> {
	let internal_id = id_public_to_db(ident).ok_or_else(|| Redirect::to(uri!("/")))?;
	let profile = doll::get(db, internal_id).await.expect("fuck");

	if let None = profile {
		return Err(Redirect::to(uri!("/")));
	}

	println!("{:?}", profile);
	Ok(Template::render(
		"show_profile",
		context! {
			profile,
		},
	))
}

#[launch]
fn rocket() -> _ {
	rocket::build()
		.attach(Template::fairing())
		.attach(DollTags::init())
		.mount("/assets", FileServer::from(relative!("/assets")))
		.mount(
			"/",
			routes![index, show_profile, show_register, handle_register],
		)
}
